;
; PGA Example, Part 1
;
; Purpose:
;    -> Set the gain of a PGA chip (PGA2310/2311/4311)
;    -> Allow control by local encoder(s) or MIDI.
;    -> Display name and levels of last change.
;
; ==========================================================================
;
; MIOS Audio Mixer
;
; (C)2007 Lyle Hazelwood (lylehaze@bellsouth.net)
; Based on example code Copyright (C) 2005  Pilo Chambert (pilo.c@wanadoo.fr)
; 
; ==========================================================================
; 
; This file is part of an MIOS example application
;
; This application is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This application is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this application; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
; ==========================================================================
;
; Required tools:
;    o Download the free available MPLAB IDE environment from the Microchip
;      Homepage http://www.microchip.com
;    o Download and install Active perl from http://www.perl.com
;    o Download and install MIDI-Ox from http://www.midiox.com
;    o Download and install the MIOS Bootstrap loader from http://www.ucapps.de and
;      
; Following steps are required to reassemble the source code:
;    o Start MPLAB IDE
;    o Open the project file (*.mcp) from this archive
;    o Open the main.asm file, here you can find all available include files 
;    o Change the files for your needs 
;    o Start "Project->Make" or just press the F10 button 
;    o After that, a new main.hex file can be found in the directory
;    o convert the main.hex file to a .syx file in the command shell with:
;      perl hex2syx.pl main.hex
;      or just click on "convert.bat"
;    o thereafter a main.syx file can be found in the directory. It
;      can be uploaded to the MIDIbox NG with MIDI-Ox (see additional
;      instructions in the MIOS bootloader archive)
;
; ==========================================================================
;; ---[ MIOS header file ]---
#include "mios.h"

;; ---[ useful macros ]---
#include "macros.h"

;; ---[ vectors to MIOS functions (never change!) ]---
#include "mios_vectors.inc"

;; ---[ user hooks (never change!) ]---
#include "user_vectors.inc"

;; ==========================================================================
;;  General Application Settings
;; ==========================================================================

;; ---[ variables used by application ]---
#include "app_defines.h"

; ==========================================================================

;; ---[ configuration table for MIDI processor and rotary encoders ]---
#include "mios_tables.inc"
	
;; lcd fucntions
#include "user_lcd.inc"

;; pga functions
#include "pga.inc"
	
#include "preset.inc"

	;MIDI volume/Pan/Balance handling
#include "setlevels.inc"

	;logarithmic conversion table	
#include "logbyte.inc"
;; ==========================================================================
;;  All MIOS hooks in one file
;; ==========================================================================



;; --------------------------------------------------------------------------
;;  This function is called by MIOS after startup to initialize the
;;  application
;; --------------------------------------------------------------------------
USER_Init
	;; define number of shift registers: for 128 IOs, we need
	;; 16 registers. Btw.: thats the maximum number of supported DIN/DOUTs
	movlw	1
	call	MIOS_SRIO_NumberSet

	;; define the update frequency (latency) of DIN/DOUTs in mS
	movlw	1 ; ms
	call	MIOS_SRIO_UpdateFrqSet

	;; Setup pga and gain to 0dB
	call	PGA_Init

;	call	PGA_LoadPreset	;removed because I don't use it.
	return


;; --------------------------------------------------------------------------
;;  This function is called by MIOS in the mainloop when nothing else is to do
;; --------------------------------------------------------------------------
USER_Tick

	return

;; --------------------------------------------------------------------------
;;  This function is periodically called by MIOS. The frequency has to be
;;  initialized with MIOS_Timer_Set
;;  Note that this is an interrupt service routine! Use FSR2 instead of FSR0
;;  and IRQ_TMPx instead of TMPx -- and make the routine as fast as possible!
;; --------------------------------------------------------------------------
USER_Timer
	return


;; --------------------------------------------------------------------------
;;  This function is called by MIOS when a debug command has been received
;;  via SysEx
;;  Input:
;;     o WREG, MIOS_PARAMETER1, MIOS_PARAMETER2, MIOS_PARAMETER3 like
;;       specified in the debug command
;;  Output:
;;     o return values WREG, MIOS_PARAMETER1, MIOS_PARAMETER2, MIOS_PARAMETER3
;; --------------------------------------------------------------------------
USER_MPROC_DebugTrigger
	return


;; --------------------------------------------------------------------------
;;  This function is called by MIOS when the display content should be
;;  initialized. Thats the case during startup and after a temporary message
;;  has been printed on the screen
;; --------------------------------------------------------------------------
USER_DISPLAY_Init
	call	MIOS_LCD_Clear			; clear LCD

	TABLE_ADDR TEXT_WELCOME_0		; print welcome message
	call	MIOS_LCD_PrintString
	TABLE_ADDR TEXT_WELCOME_1		; print welcome message
	call	MIOS_LCD_PrintString

	return
; Here's my tacky startup text
TEXT_WELCOME_0	STRING 20, 0x00, "  MIOS Audio Mixer  "
TEXT_WELCOME_1	STRING 20, 0x40, "   Lyle Hazelwood   "


;; --------------------------------------------------------------------------
;;  This function is called by MIOS when a complete MIDI event has been received
;;  Input:
;;     o first  MIDI event byte in MIOS_PARAMETER1
;;     o second MIDI event byte in MIOS_PARAMETER2
;;     o third  MIDI event byte in MIOS_PARAMETER3
;; --------------------------------------------------------------------------
;We simply look for CC#7(Volume) and CC#8(Balance) messages
USER_MPROC_NotifyReceivedEvent
	movf	MIOS_PARAMETER1,W	;get status byte
	andlw	0xF0				;mask out low half (ignore channel info)
	sublw	0xB0				;compare to control change?
	bz		contchg					
	return						;not a control change, so return
contchg								;if we get this far, we know it's SOME kind of control change
	movlw	7					;Volume CC #7
	cpfseq	MIOS_PARAMETER2		;skip the next line if it IS a volume message
	bra		ck4bal				;no volume, check for balance/pan
	lfsr	FSR0,MIDI_VOL_0		;point to MIDI volumes table
	movf	MIOS_PARAMETER1,W	;get the status byte again
	andlw	0x0F				;mask for just the channel number
	movwf	MIDI_CC_CHAN		;capture our channel number
	movwf	MIDI_DISP_CHAN		;show this channel on the display
	addwf	FSR0L				;offset into MIDI_VOL table
	movf	MIOS_PARAMETER3,W	;get the new value into W
	movwf	INDF0				;and write it to the table
	call	Mid2Gain			;convert all MIDI settings into gain settings
	call	PGA_SEND_GAIN		;and send those settings out to the chips
	bsf		DISPLAY_UPDATE_REQ,0	;update the display
	return
ck4bal
	movlw	8					;MIDI Balance
	cpfseq	MIOS_PARAMETER2
	bra		ck4pan
	bra		dobalance
ck4pan
	movlw	10
	cpfseq	MIOS_PARAMETER2
	return						;It's not a volume, balance, or Pan control, let it go.
dobalance							;once we are here, we have either a Balance or Pan control
	lfsr	FSR0,MIDI_BAL_0		;point to the start of the balance table
	movf	MIOS_PARAMETER1,W	
	andlw	0x0F				;get the channel number
	movwf	MIDI_CC_CHAN
	movwf	MIDI_DISP_CHAN
	addwf	FSR0L				;add it to the table pointer
	movf	MIOS_PARAMETER3,W	;get the new setting
	movwf	INDF0				;and copy it to the table
	call	Mid2Gain			;convert all MIDI settings to gain settings
	call 	PGA_SEND_GAIN		;send them to the chips
	bsf		DISPLAY_UPDATE_REQ,0	;and update the display
	return

;; --------------------------------------------------------------------------
;;  This function is called by MIOS when a MIDI event has been received
;;  which has been specified in the MIOS_MPROC_EVENT_TABLE table
;;  Input:
;;     o number of entry in WREG
;;     o first  MIDI event byte in MIOS_PARAMETER1
;;     o second MIDI event byte in MIOS_PARAMETER2
;;     o third  MIDI event byte in MIOS_PARAMETER3
;; --------------------------------------------------------------------------
USER_MPROC_NotifyFoundEvent
	return


;; --------------------------------------------------------------------------
;;  This function is called by MIOS when a MIDI event has not been completly
;;  received within 2 seconds
;; --------------------------------------------------------------------------
USER_MPROC_NotifyTimeout
	return


;; --------------------------------------------------------------------------
;;  This function is called by MIOS when a MIDI byte has been received
;;  Input:
;;     o received MIDI byte in WREG and MIOS_PARAMETER1
;; --------------------------------------------------------------------------
USER_MPROC_NotifyReceivedByte
	return


;; --------------------------------------------------------------------------
;;  This function is called by MIOS when an button has been toggled
;;  Input:
;;     o Button number in WREG and MIOS_PARAMETER1
;;     o Button value MIOS_PARAMETER2:
;;       - 1 if button has been released (=5V)
;;       - 0 if button has been pressed (=0V)
;; --------------------------------------------------------------------------
USER_DIN_NotifyToggle

	btfss	MIOS_PARAMETER2,0	;skip next line if button released
	return						;return if the button was pressed
	bsf	DISPLAY_UPDATE_REQ, 0	;update the display
	incf	MIDI_DISP_CHAN,f	;add one to the display channel
	movlw	pairs-1
	cpfsgt	MIDI_DISP_CHAN		;if the channel number is greater than pairs,
	return
	clrf	MIDI_DISP_CHAN		;reset to zero
	return

;; --------------------------------------------------------------------------
;;  This function is called by MIOS when an encoder has been moved
;;  Input:
;;     o Encoder number in WREG and MIOS_PARAMETER1
;;     o signed incrementer value in MIOS_PARAMETER2:
;;       - is positive when encoder has been turned clockwise
;;       - is negative when encoder has been turned counter clockwise
;; --------------------------------------------------------------------------
USER_ENC_NotifyChange
	;; an encoder has been moved. 

	;; save encoder number in ENC_NUMBER
	movff	MIOS_PARAMETER1, ENC_NUMBER
	;; save incrementer in ENC_INC
	movff	MIOS_PARAMETER2, ENC_INC

	rgoto	USER_ENC_GAIN

;;--------------------------------------------------------------------------
USER_ENC_GAIN
	;; inc or dec the value store at CHANNEL_GAIN_0 + CHANNEL_NUMBER
	;; pointer = CHANNEL_GAIN_0 + CHANNEL_NUMBER
	bcf		DISPLAY_UPDATE_REQ,7
	movlw	d'2'				; the pin number for our pushbutton
	call	MIOS_DIN_PinGet		;check to see if button is pushed
	btfsc	WREG,0
	bsf		DISPLAY_UPDATE_REQ,7

	lfsr	FSR0, MIDI_VOL_0		;point to MIDI Volume
	btfss	DISPLAY_UPDATE_REQ,7	;or maybe
	lfsr	FSR0,MIDI_BAL_0			;MIDI Balance table

	movf	MIDI_DISP_CHAN, W		;add the selected channel offset
 	addwf	FSR0L, F	; FSR0H is already set by lfsr FSR0,

	;; branch depending on clockwise/counter clockwise turn
 	btfsc	MIOS_PARAMETER2, 7
	rgoto	USER_ENC_Handler_Para_Dec

USER_ENC_Handler_Para_Inc
	;; add incrementer to current parameter value
	movf	MIOS_PARAMETER2, W
	addwf	INDF0, F
	;; continue if value <= 0x7f
	movlw	0x7f
	cpfsgt	INDF0
	rgoto USER_ENC_Handler_Para_Cont
	;; else set max value
	movlw	0x7f
	movwf	INDF0
	rgoto	USER_ENC_Handler_Para_Cont

USER_ENC_Handler_Para_Dec
	;; subtract incrementer from current parameter value
	movf	MIOS_PARAMETER2, W	; (it's a negative value...)
	addwf	INDF0, F
	;; continue if no overflow
	bc	USER_ENC_Handler_Para_Cont
	;; else set value to zero
	clrf	INDF0
	;; 	rgoto	TC_ENC_Handler_Para_Cont

USER_ENC_Handler_Para_Cont
				;this will send the change out the MIDI Port.
	call	MIOS_MIDI_BeginStream
	movlw	0xB0
	addwf	MIDI_DISP_CHAN,W
	call	MIOS_MIDI_TxBufferPut
	movlw	0x07
	btfss	DISPLAY_UPDATE_REQ,7
	movlw	0x0a
	call	MIOS_MIDI_TxBufferPut
	movf	INDF0,W
	call	MIOS_MIDI_TxBufferPut
	call	MIOS_MIDI_EndStream

	bsf		DISPLAY_UPDATE_REQ, 0	;update the display
	call 	Mid2Gain
	call	PGA_SEND_GAIN
;	call	PGA_SavePreset	;I don't want to burn Flash every time I change the mix.
	return

;; --------------------------------------------------------------------------
;;  This function is called by MIOS before the shift register are loaded
;;  Note that this is an interrupt service routine! Use FSR2 instead of FSR0
;;  and IRQ_TMPx instead of TMPx -- and make the routine as fast as possible
;; --------------------------------------------------------------------------
USER_SR_Service_Prepare
	return


;; --------------------------------------------------------------------------
;;  This function is called by MIOS after the shift register have been loaded
;;  Note that this is an interrupt service routine! Use FSR2 instead of FSR0
;;  and IRQ_TMPx instead of TMPx -- and make the routine as fast as possible
;; --------------------------------------------------------------------------
USER_SR_Service_Finish
	return

;; --------------------------------------------------------------------------
;;  This function is called by MIOS when a pot has been moved
;;  Input:
;;     o Pot number in WREG and MIOS_PARAMETER1
;;     o LSB value in MIOS_PARAMETER2
;;     o MSB value in MIOS_PARAMETER3
;; --------------------------------------------------------------------------
USER_AIN_NotifyChange
	return

	END
