/*
 * MIOS Pedal Box / Pedal Board - main.c
 * vbeta1.16 - September 2007
 * ==========================================================================
 *
 *  Copyright (C) 2007  Mick Crozier mcrozier@iinet.net.au
 *  Licensed for personal non-commercial use only.
 *  All other rights reserved.
 *
 * ==========================================================================
 */


#include "cmios.h"
#include "pic18f452.h"

#include "main.h"
#include "pbx.c"
#if DIGITS_CONNECTED
  #include "pbx_digits.c"
#endif
#include "pbx_lcd.c"

/////////////////////////////////////////////////////////////////////////////
// This function is called by MIOS after startup to initialize the
// application
/////////////////////////////////////////////////////////////////////////////
void Init(void) __wparam
{

      // initialize the AIN driver
  MIOS_AIN_NumberSet(AIN_NUMBER_INPUTS);  // 8 pots are connected
  MIOS_AIN_UnMuxed();    // no AINX4 modules are used
  MIOS_AIN_DeadbandSet(AIN_DEADBAND); // should be 7 when 7bit resolution is used


      //Enable SRIO
      // set shift register update frequency
    MIOS_SRIO_UpdateFrqSet(1); // ms

    // we need to set at least one IO shift register pair
    MIOS_SRIO_NumberSet(NUMBER_OF_SRIO);

    // debouncing value for DINs
    MIOS_SRIO_DebounceSet(DIN_DEBOUNCE_VALUE);


  //Enable MIDI Merger
  #if MIDI_MERGER
    MIOS_MIDI_MergerSet(MIOS_MIDI_MERGER_ENABLED);
  #else
    MIOS_MIDI_MergerSet(MIOS_MIDI_MERGER_DISABLED);
  #endif

 // MIOS_BANKSTICK_CtrlSet(0x00); // Set bankstick


//Load some data from bankstick
  for (t=0; t<16; t++) device_name[t] = MIOS_BANKSTICK_Read(0x6800 + t);
  for (t=0; t<8; t++) AIN_fixed[t] = MIOS_BANKSTICK_Read(0x6900 + t);
  //

  for (t=0; t<8; t++) {
    MIOS_BANKSTICK_CtrlSet(t);
    bankstick_channel[t] = MIOS_BANKSTICK_Read(0x6ffe);
    if( !MIOS_BOX_STAT.BS_AVAILABLE ) {
      bankstick_channel[t] = 255; // will be used as error code for no bankstick
    }
  }
  for (t=0; t<152; t++) {
    bankstick_map[t] = MIOS_EEPROM_Read(t);
    for (i=0; i<8; i++) {
      if (bankstick_channel[i] == 255 && bankstick_map[t] == i) bankstick_map[i] = 0; // if bankstick not present, use first bankstick
    }
  }


  #if DIGITS_CONNECTED
  digits_handler(bank); // display current bank
  #endif

  // Set Variables
  for (t=0; t<8; t++) {
    //current_programchange_evnt1[t] = 0x00;
    current_programchange_evnt1[t] = 0x00;
    event_current_programchange[t] = 128;
  }
  bank = 1;
  for (t=0; t<144; t++) event_current_value[t] = 0x00;  // MAX BUTTONS 16 fixed commands, 128 in banked commands
  program_mode = NONE;
  previous_pin_select = 255;
  count = 0;

}

/////////////////////////////////////////////////////////////////////////////
// This function is called by MIOS in the mainloop when nothing else is to do
/////////////////////////////////////////////////////////////////////////////
void Tick(void) __wparam
{
}

/////////////////////////////////////////////////////////////////////////////
// This function is periodically called by MIOS. The frequency has to be
// initialized with MIOS_Timer_Set
/////////////////////////////////////////////////////////////////////////////
void Timer(void) __wparam
{
    if (app_flags.PCRX == 1) {
      countms++;
      if (countms = 10) {
        app_flags.PCRX = 0;
        MIOS_TIMER_Stop();
      }
    } else {
      bpmcountms++; //add 1 to bpmcountms
      if (bpmcountms > 3000 ) { //3 seconds have passed
        tap_tempo_stop();
      }
    }

}

/////////////////////////////////////////////////////////////////////////////
// This function is called by MIOS when the display content should be
// initialized. Thats the case during startup and after a temporary message
// has been printed on the screen
/////////////////////////////////////////////////////////////////////////////
void DISPLAY_Init(void) __wparam
{
  if (app_flags.SPLASH == 0) {
    MIOS_LCD_Clear();
    MIOS_LCD_CursorSet(0x00);
    #if PEDALBOARD
      MIOS_LCD_PrintCString("Pedal Board B116");
    #else
      MIOS_LCD_PrintCString("Pedal Box B1.16 ");
    #endif
    MIOS_LCD_CursorSet(0x40);
    for (t = 0; t<16; t++) MIOS_LCD_PrintChar(device_name[t]);
    app_flags.SPLASH = 1;
  } else {
    app_flags.DISPLAY_UPDATE_REQ = 1;
  }
}

/////////////////////////////////////////////////////////////////////////////
//  This function is called in the mainloop when no temporary message is shown
//  on screen. Print the realtime messages here
/////////////////////////////////////////////////////////////////////////////
void DISPLAY_Tick(void) __wparam
{
// do nothing if no update has been requested
  if( !app_flags.DISPLAY_UPDATE_REQ )
    return;

  display_handler();

#if DIGITS_CONNECTED
    digits_handler(bank); // update bank display
#endif
#if PEDALBOARD
  set_led_indicators(bank); //update led indicators
#endif
}


/////////////////////////////////////////////////////////////////////////////
//  This function is called by MIOS when a complete MIDI event has been received
/////////////////////////////////////////////////////////////////////////////
void MPROC_NotifyReceivedEvnt(unsigned char evnt0, unsigned char evnt1, unsigned char evnt2) __wparam
{

// Start with setup and misc functions
  if (evnt0 == EVENT_SETUP_E0 && evnt1 == EVENT_SETUP_E1 && evnt2 == EVENT_SETUP_E2) {

    if (program_mode == NONE) {
      //program_mode_handler; -- THIS SHOULD WORK, BUT DOESN'T, CODE TEMPORARILY ADDED HERE
      bankstick_select = bankstick_map[pin_select];
      MIOS_BANKSTICK_CtrlSet(bankstick_select);
      channel = MIOS_BANKSTICK_Read(0x6ffe);
      MIOS_AIN_DeadbandSet(5);
      program_mode = EVENT_SELECT;  // in the display handler, program mode is set to 2
      MIOS_LCD_MessageStop();
      app_flags.DISPLAY_UPDATE_REQ = 1;
    } else {
      program_mode_handler();
    }
    MIOS_LCD_MessageStop();
    app_flags.DISPLAY_UPDATE_REQ = 1;
  }

  else if (evnt0 == SCREEN_CHANGE_E0 && evnt1 == SCREEN_CHANGE_E1 && evnt2 == SCREEN_CHANGE_E2 && program_mode != NONE) {

    if (program_mode == EVENT_SELECT) {
      program_mode = SET_MIN;
    }
    else if (program_mode == SET_MIN) {
      program_mode = SET_MAX;
    }
    else if (program_mode == SET_MAX) {
      program_mode = SET_BANKSTICK;
    }
    else if (program_mode == SET_BANKSTICK) {
      program_mode = EVENT_SELECT;
    }
    else if (program_mode == SET_AIN_FIXED) {
      program_mode = SET_BANKSTICK1_CH;
    }
    else if (program_mode >= SET_BANKSTICK1_CH && program_mode < SET_BANKSTICK8_CH) {
      program_mode++;
    }
    else if (program_mode == SET_BANKSTICK8_CH) {
      program_mode = SET_AIN_FIXED;
    }
    MIOS_LCD_MessageStop();
    app_flags.DISPLAY_UPDATE_REQ = 1;
  }

  else if (evnt0 == GLOBAL_SETUP_E0 && evnt1 == GLOBAL_SETUP_E1 && evnt2 == GLOBAL_SETUP_E2) {

    if (program_mode == NONE) {
      program_mode = SET_AIN_FIXED;
    }
    MIOS_LCD_MessageStop();
    app_flags.DISPLAY_UPDATE_REQ = 1;
  }


     // Normal MIDI mode

  else if (program_mode == NONE) {
    if (app_flags.PCRX == 0) {
    // Scan the event_map, returns the entry number and fills found_event with data from table
    found_event.entry = scan_event_map(evnt0, evnt1);
    found_event.param2 = evnt2;

    if(found_event.entry != 0xffff) { //if matching event is found
      if( found_event.event_handler == TAP_TEMPO ) { // is tap tempo button
        tap_tempo_handler();
        display_temp();
      }

      else if ( found_event.event_handler == ON_OFF_ONLY ) { // toggle button
        #if PEDALBOARD
        event_current_value[found_event.entry] = evnt2;// set current value
        #endif
        display_temp();
      }

      else if ( found_event.event_handler == PROGRAM_CHANGE ) { // program change
    // set the program change indicator
        event_previous_programchange[MIOS_BANKSTICK_CtrlGet()] =  event_current_programchange[MIOS_BANKSTICK_CtrlGet()];
        event_current_programchange[MIOS_BANKSTICK_CtrlGet()] = found_event.entry;
        current_programchange_evnt1[MIOS_BANKSTICK_CtrlGet()] = found_event.param1;
        if (event_current_programchange!=event_previous_programchange) {
          event_current_value[event_previous_programchange[MIOS_BANKSTICK_CtrlGet()]] = 0;
          event_current_value[event_current_programchange[MIOS_BANKSTICK_CtrlGet()]] = 1;
        } else {
          event_current_value[event_current_programchange[MIOS_BANKSTICK_CtrlGet()]] = 1;
        }
        MIOS_LCD_MessageStop();
        app_flags.DISPLAY_UPDATE_REQ = 1;

        // This prevents midi messages associated with a program change from being shown on the display
        // by suspending midi handling (except midi merge) for 10ms. (uses the MIOS timer).
        countms = 0;
        app_flags.PCRX = 1;
        MIOS_TIMER_Init(0x00, 10000); // first tap press

      }
      else if ( found_event.event_handler >= METER_ONLY ) { // Usually pedal functions
        display_temp();
      }
    }
    else { // if found_event.entry = 0xffff (not found error code)
      // Do Nothing
    }
    } // app_flags.PCRX = 0
  } // progran mode = NONE
}

/////////////////////////////////////////////////////////////////////////////
// This function is called by MIOS when a MIDI event has been received
// which has been specified in the MIOS_MPROC_EVENT_TABLE
/////////////////////////////////////////////////////////////////////////////
void MPROC_NotifyFoundEvent(unsigned entry, unsigned char evnt0, unsigned char evnt1, unsigned char evnt2) __wparam
{
}

/////////////////////////////////////////////////////////////////////////////
// This function is called by MIOS when a MIDI event has not been completly
// received within 2 seconds
/////////////////////////////////////////////////////////////////////////////
void MPROC_NotifyTimeout(void) __wparam
{
}

/////////////////////////////////////////////////////////////////////////////
// This function is called by MIOS when a MIDI byte has been received
/////////////////////////////////////////////////////////////////////////////
void MPROC_NotifyReceivedByte(unsigned char byte) __wparam
{
}

/////////////////////////////////////////////////////////////////////////////
// This function is called by MIOS before the shift register are loaded
/////////////////////////////////////////////////////////////////////////////
void SR_Service_Prepare(void) __wparam
{
}

/////////////////////////////////////////////////////////////////////////////
// This function is called by MIOS after the shift register have been loaded
/////////////////////////////////////////////////////////////////////////////
void SR_Service_Finish(void) __wparam
{
}

/////////////////////////////////////////////////////////////////////////////
// This function is called by MIOS when an button has been toggled
// pin_value is 1 when button released, and 0 when button pressed
// If floorboard mode not active, this is irrelevant.
/////////////////////////////////////////////////////////////////////////////
void DIN_NotifyToggle(unsigned char pin, unsigned char pin_value) __wparam
{


 // Start with handling of special function buttons

  if (pin == EVENT_SETUP_DIN_PIN && pin_value == 0 && MIOS_DIN_PinGet(DIN_BANK_DOWN) == 0 && program_mode == NONE) {
      //program_mode_handler; -- THIS SHOULD WORK, BUT DOESN'T, CODE TEMPORARILY ADDED HERE
      bankstick_select = bankstick_map[pin_select];
      MIOS_BANKSTICK_CtrlSet(bankstick_select);
      channel = MIOS_BANKSTICK_Read(0x6ffe);
      MIOS_AIN_DeadbandSet(5);
      program_mode = EVENT_SELECT;  // in the display handler, program mode is set to 2
      MIOS_LCD_MessageStop();
      app_flags.DISPLAY_UPDATE_REQ = 1;
  }

  else if (pin == GLOBAL_SETUP_DIN_PIN && pin_value == 0 && MIOS_DIN_PinGet(DIN_BANK_DOWN) == 0 && program_mode == NONE) {
      program_mode = SET_AIN_FIXED;
      MIOS_LCD_MessageStop();
      app_flags.DISPLAY_UPDATE_REQ = 1;
  }

  else if (pin == DIN_BANK_UP && pin_value == 0) {

      if (program_mode == EVENT_SELECT) {
        program_mode = SET_MIN;
        MIOS_LCD_MessageStop();
        app_flags.DISPLAY_UPDATE_REQ = 1;
      }
      else if (program_mode == SET_MIN) {
        program_mode = SET_MAX;
        MIOS_LCD_MessageStop();
        app_flags.DISPLAY_UPDATE_REQ = 1;
      }
      else if (program_mode == SET_MAX) {
        program_mode = SET_BANKSTICK;
        MIOS_LCD_MessageStop();
        app_flags.DISPLAY_UPDATE_REQ = 1;
      }
      else if (program_mode == SET_BANKSTICK) {
        program_mode = EVENT_SELECT;
        MIOS_LCD_MessageStop();
        app_flags.DISPLAY_UPDATE_REQ = 1;
      }
      else if (program_mode == SET_AIN_FIXED) {
        program_mode = SET_BANKSTICK1_CH;
        MIOS_LCD_MessageStop();
        app_flags.DISPLAY_UPDATE_REQ = 1;
      }
      else if (program_mode >= SET_BANKSTICK1_CH && program_mode < SET_BANKSTICK8_CH) {
        program_mode++;
        MIOS_LCD_MessageStop();
        app_flags.DISPLAY_UPDATE_REQ = 1;
      }
      else if (program_mode == SET_BANKSTICK8_CH) {
        program_mode = SET_AIN_FIXED;
        MIOS_LCD_MessageStop();
        app_flags.DISPLAY_UPDATE_REQ = 1;
      }
      else if (program_mode == NONE) {
        bank++;
        if (bank > (128 / DIN_BANKED_BUTTONS) + 1 ) bank = (128 / DIN_BANKED_BUTTONS) + 1 ; // Max bank = (128 / the amount of banked buttons) + 1
        MIOS_LCD_MessageStop();
        app_flags.DISPLAY_UPDATE_REQ = 1;
     }
  }

  else if (pin == DIN_BANK_DOWN && pin_value == 0) {

    if (program_mode > NONE) {
      program_mode_handler();
      MIOS_LCD_MessageStop();
      app_flags.DISPLAY_UPDATE_REQ = 1;
    }
    else {
      bank --;
      if (bank < 1) bank = 1;
      MIOS_LCD_MessageStop();
      app_flags.DISPLAY_UPDATE_REQ = 1;
    }
  }

  else if (program_mode == SET_MIN || program_mode == SET_MAX && pin_value == 0) {

    if (pin == EVENT_SETUP_DIN_PIN) { // for switch/momentary toggling
      if (button_type == 0) {
        button_type = 1;
      } else {
        button_type = 0;
      }
      MIOS_LCD_MessageStop();
      app_flags.DISPLAY_UPDATE_REQ = 1;
    }
  }

  else if (program_mode == NONE) { // Handling for floorboard mode

#if PEDALBOARD
      if (pin > DIN_FIXED_BUTTONS - 1) {
        pin_select = pin + (bank * DIN_BANKED_BUTTONS) - DIN_BANKED_BUTTONS - DIN_FIXED_BUTTONS + 16;
        if (pin_select > 143) pin_select = 143; // catch for the 'make up' bank to stop buttons with values higher than 128
      } else {
        pin_select = pin;
      }
    bankstick_select = bankstick_map[pin_select];
    MIOS_BANKSTICK_CtrlSet(bankstick_select);
    channel = MIOS_BANKSTICK_Read(0x6ffe);

      pin_mapped = MIOS_BANKSTICK_Read(0x6520 + (pin_select * 4));
      low_value = MIOS_BANKSTICK_Read(0x6520 + (pin_select * 4) + 1);
      high_value = MIOS_BANKSTICK_Read(0x6520 + (pin_select * 4) + 2);
      button_type = MIOS_BANKSTICK_Read(0x6520 + (pin_select * 4) + 3);
      pin_mapto = pin_mapped; // used by programing mode

    // Set found_event variables
      for (t=0; t<16; t++) found_event.name[t] = MIOS_BANKSTICK_Read(0x0000 + (pin_mapped * 20) + t);
      found_event.status = MIOS_BANKSTICK_Read(0x0000 + (pin_mapped * 20) + 16);
      found_event.param1 = MIOS_BANKSTICK_Read(0x0000 + (pin_mapped * 20) + 17);
      found_event.event_handler = MIOS_BANKSTICK_Read(0x0000 + (pin_mapped * 20) + 18);

    // Button handling dependant on type (event_handler)
      if (found_event.event_handler == ON_OFF_ONLY) {  // is an on/off funtion
        if (pin_value == 0 && button_type == 0) { // is toggle type
          if (event_current_value[pin_mapped] == 0x00) {
            found_event.param2 = 0x7f;
          } else {
            found_event.param2 = 0x00;
          }
          event_current_value[pin_mapped] = dout_toggle(found_event.param2, low_value);
          MIDI_message_TX(found_event.status, found_event.param1, found_event.param2, channel);
          display_temp();
        } else if (pin_value == 1 && button_type == 1) { // is momentary off
          found_event.param2 = 0x00;
          event_current_value[pin_mapped] = dout_toggle(found_event.param2, low_value);
          MIDI_message_TX(found_event.status, found_event.param1, found_event.param2, channel);
          display_temp();
        } else if (pin_value == 0 && button_type == 1) { // is momentary on
          found_event.param2 = 0x7f;
          event_current_value[pin_mapped] = dout_toggle(found_event.param2, low_value);
          MIDI_message_TX(found_event.status, found_event.param1, found_event.param2, channel);
          display_temp();
        }

      } else if (found_event.event_handler == PROGRAM_CHANGE && pin_value == 0) { // is a program change function
        event_previous_programchange[MIOS_BANKSTICK_CtrlGet()] = event_current_programchange[MIOS_BANKSTICK_CtrlGet()]; // program change pin number
        event_current_programchange[MIOS_BANKSTICK_CtrlGet()] = pin_mapped;
        current_programchange_evnt1[MIOS_BANKSTICK_CtrlGet()] = found_event.param1;
        event_current_value[event_previous_programchange[MIOS_BANKSTICK_CtrlGet()]] = 0;
        event_current_value[event_current_programchange[MIOS_BANKSTICK_CtrlGet()]] = 1; // settings for leds
        found_event.param2 = 0x7f;
        current_programchange_evnt1[MIOS_BANKSTICK_CtrlGet()] = found_event.param1; // program change number in hex
        MIDI_message_TX(found_event.status, found_event.param1, found_event.param2, channel);
        MIOS_LCD_MessageStop();
        app_flags.DISPLAY_UPDATE_REQ = 1;

      } else if(found_event.event_handler == TAP_TEMPO) { // is the tap tempo function
        if (pin_value == 0) {
          tap_tempo_handler();
          event_current_value[pin_mapped] = 1; // force led on
          MIDI_message_TX(found_event.status, found_event.param1, found_event.param2, channel);
          display_temp();
        } else {
          event_current_value[pin_mapped] = 0; // force led off
          set_led_indicators(bank);
        }

      } else {        // > 2  - More handling could go here if needed
        if (pin_value == 0 && button_type == 0) { // is toggle type
          if (event_current_value[pin_mapped] == 0) {
            found_event.param2 = high_value;
          } else {
            found_event.param2 = low_value;
          }
          event_current_value[pin_mapped] = dout_toggle(found_event.param2, low_value);
          MIDI_message_TX(found_event.status, found_event.param1, found_event.param2, channel);
          display_temp();
        } else if (pin_value == 1 && button_type == 1) { // is momentary off
          found_event.param2 = low_value;
          event_current_value[pin_mapped] = dout_toggle(found_event.param2, low_value);
          MIDI_message_TX(found_event.status, found_event.param1, found_event.param2, channel);
          display_temp();
        } else if (pin_value == 0 && button_type == 1) { // is momentary on
          found_event.param2 = high_value;
          event_current_value[pin_mapped] = dout_toggle(found_event.param2, low_value);
          MIDI_message_TX(found_event.status, found_event.param1, found_event.param2, channel);
          display_temp();
        }
      }
      #endif //PEDALBOARD
  }

}

/////////////////////////////////////////////////////////////////////////////
// This function is called by MIOS when an encoder has been moved
// incrementer is positive when encoder has been turned clockwise, else
// it is negative
/////////////////////////////////////////////////////////////////////////////
void ENC_NotifyChange(unsigned char encoder, char incrementer) __wparam
{
}

/////////////////////////////////////////////////////////////////////////////
// This function is called by MIOS when a pot has been moved
/////////////////////////////////////////////////////////////////////////////
void AIN_NotifyChange(unsigned char pin, unsigned int pin_value) __wparam
{
  // Start with program mode handler
  if (program_mode == EVENT_SELECT) {  // In program mode

      pin_mapto = (MIOS_AIN_PinGet(pin)) >> 2; // gives entry number between 0-255
      app_flags.DISPLAY_UPDATE_REQ = 1;
  }

  else if (program_mode == SET_MIN)  {

      low_value = MIOS_AIN_Pin7bitGet(pin);
      app_flags.DISPLAY_UPDATE_REQ = 1;
  }

  else if (program_mode == SET_MAX) {

      high_value = MIOS_AIN_Pin7bitGet(pin);
      app_flags.DISPLAY_UPDATE_REQ = 1;
  }

  else if (program_mode == SET_AIN_FIXED)  {

    AIN_fixed[pin] = MIOS_AIN_Pin7bitGet(pin) >> 6; // gives 0 or 1
    app_flags.DISPLAY_UPDATE_REQ = 1;
  }
  else if (program_mode == SET_BANKSTICK) {
    bankstick_select = MIOS_AIN_Pin7bitGet(pin) >> 4;
    app_flags.DISPLAY_UPDATE_REQ = 1;
  }
  else if (program_mode >= SET_BANKSTICK1_CH) {
    if (bankstick_channel[program_mode - SET_BANKSTICK1_CH] != 255) {
      bankstick_channel[program_mode - SET_BANKSTICK1_CH] = MIOS_AIN_Pin7bitGet(pin) >> 3;
    }
    app_flags.DISPLAY_UPDATE_REQ = 1;
  }

  // Handling for Normal MIDI mode
  else {
    pin_select = pin + 144; // used by programing mode to identify AIN pins
    bankstick_select = bankstick_map[pin_select];
    MIOS_BANKSTICK_CtrlSet(bankstick_select);
    channel = MIOS_BANKSTICK_Read(0x6ffe);

    AIN_map_offset = pin*0x200;
    if (AIN_fixed[pin] == 0) { // pin st to change midi event with program change
      pin_mapped = MIOS_BANKSTICK_Read(0x7000 + (AIN_map_offset) + (current_programchange_evnt1[MIOS_BANKSTICK_CtrlGet()] * 4)); // get the assigned midi function
      low_value = MIOS_BANKSTICK_Read(0x7000 + (AIN_map_offset) + (current_programchange_evnt1[MIOS_BANKSTICK_CtrlGet()] * 4) + 1);
      high_value = MIOS_BANKSTICK_Read(0x7000 + (AIN_map_offset) + (current_programchange_evnt1[MIOS_BANKSTICK_CtrlGet()] * 4) + 2);
    } else {
      pin_mapped = MIOS_BANKSTICK_Read(0x7000 + (AIN_map_offset)); // get the assigned midi function
      low_value = MIOS_BANKSTICK_Read(0x7000 + (AIN_map_offset) + 1);
      high_value = MIOS_BANKSTICK_Read(0x7000 + (AIN_map_offset) + 2);
    }
    pin_mapto = pin_mapped; // used by programing mode, displays current setting

    // Set found_event variables
    for (t=0; t<16; t++) found_event.name[t] = MIOS_BANKSTICK_Read(0x0000 + (pin_mapped * 20) + t);
    found_event.status = MIOS_BANKSTICK_Read(0x0000 + (pin_mapped * 20) + 16);
    found_event.param1 = MIOS_BANKSTICK_Read(0x0000 + (pin_mapped * 20) + 17);
    found_event.event_handler = MIOS_BANKSTICK_Read(0x0000 + (pin_mapped * 20) + 18);
    found_event.param2 = Scale_7bit(MIOS_AIN_Pin7bitGet(pin), low_value, high_value);

    display_temp();

  // send mapped midi event
    MIDI_message_TX(found_event.status, found_event.param1, found_event.param2, channel);

  }
}
