/*
 * MIOS Pedal Box / Pedal Board - main.h
 * vbeta1.16 - September 2007
 * ==========================================================================
 *
 *  Copyright (C) 2007  Mick Crozier mcrozier@iinet.net.au
 *  Licensed for personal non-commercial use only.
 *  All other rights reserved.
 *
 * ==========================================================================
 */

///////////////////////////////////////////////////////////////////////////
// Global Parameters
//
///////////////////////////////////////////////////////////////////////////

#define   PEDALBOARD             1  // 1 = Pedal Board mode    0 = Pedal Box mode
#define   MIDI_MERGER            0  // 0 = off   1 = on

 // Input Parameters
#define   AIN_DEADBAND           7 // 7 for 7-bit midi is best
#define   AIN_NUMBER_INPUTS      8 // 1 - 8 - number of pots connected

#define   NUMBER_OF_SRIO         3 // 1-16  number of shift registers connected (count DIN or DOUT, whichever has more)
#define   DIN_DEBOUNCE_VALUE     20 // debounce value

// FIXED BUTTONS MUST CONNECT FIRST
#define   DIN_FIXED_BUTTONS      4 // 0-16 buttons that always stay the same, regardless of bank

// BANKED BUTTONS MUST BE CONNECTED AFTER FIXED BUTTONS
#define   DIN_BANKED_BUTTONS     2 // 0-16 buttons that change with bank. 1 banked button will cause display errors when bank is over 99

 //THESE SPECIAL FUNCTION BUTTONS MUST BE CONNECTED AFTER THE BANKED BUTTONS!!!!!!!
#define   DIN_BANK_UP            6   // 2 - 34, pin number of button used to bank up
#define   DIN_BANK_DOWN          7   // 2 - 34, pin number of button used to bank down

#define   EVENT_SETUP_DIN_PIN    0   // 0 - 34, pin number of button used to enter event setup
#define   GLOBAL_SETUP_DIN_PIN   1   // 0 - 34, pin number of button used to enter AIN setup

// Enter setup with MIDI event
#define   EVENT_SETUP_E0        0xb0 // 0x90 - 0xef - Status
#define   EVENT_SETUP_E1        0x7f // 0x00 - 0x7f - Param1
#define   EVENT_SETUP_E2        0x7f // 0x00 - 0x7f - Param2

// AIN setup with MIDI event
#define   GLOBAL_SETUP_E0       0xb0 // 0x90 - 0xef - Status
#define   GLOBAL_SETUP_E1       0x7f // 0x00 - 0x7f - Param1
#define   GLOBAL_SETUP_E2       0x7d // 0x00 - 0x7f - Param2

// Scrren change with MIDI event
#define   SCREEN_CHANGE_E0      0xb0 // 0x90 - 0xef - Status Byte
#define   SCREEN_CHANGE_E1      0x7f // 0x00 - 0x7f - Param1 Byte
#define   SCREEN_CHANGE_E2      0x7e // 0x00 - 0x7f - Param2 Byte

  //LED Digit Setup (Floorboard mode only)
#define   DIGITS_CONNECTED       2  // 0-2 How many LED digits are connected
#define   DOUT_DIGITS_START      12  // 0,8,16,24,32 What DOUT pin do the digits start from.
                                    // Must be multiple of 8 and higher than DIN_FIXED_BUTTONS
                                    // Or conflict with LED indicator handler will occur




 ///////////////////////////////////////////////////////////////////////////
 // Definitions
 //
 ///////////////////////////////////////////////////////////////////////////

typedef union {
  struct {
    unsigned ALL:8;
  };
  struct {
    unsigned DISPLAY_UPDATE_REQ:1;  // requests a display update
    unsigned TAP_FLAG:1;            // Tells tap tempo timer to start/stop
    unsigned SPLASH:1;
    unsigned PCRX:1;                // Recived program change
  };
} app_flags_t;


typedef struct found_event_format
{
  unsigned char name[17];
  unsigned char status;
  unsigned char param1;
  unsigned char event_handler; // defines the midi function type (read more in pbx_tables.c)
  unsigned char param2;
  unsigned int entry; // entry number in event map
} found_event_t;

///////////////////////////////////////////////////////////////////////////
// Variables
//
///////////////////////////////////////////////////////////////////////////

// status of application
app_flags_t app_flags;

// place holder for found event // used by application for incoming and outgoing midi events
found_event_t found_event;

unsigned char program_mode; // What mode 0 = normal/none, 1 = enter setup, 2 = select function, 3 = low/high/type setup, 4 = AIN fixed settings, 5 = exit

//used by visualization
unsigned char param2_value_reduced;

//used in 'for' loops and as temp variables for calculations
unsigned char t;
unsigned short int i;

//used for bpm calculations
unsigned short int bpm;
unsigned short int bpmcountms = 0;


unsigned char event_current_value[256]; // Stores 1 for ON and 0 for OFF
unsigned char event_previous_programchange[8]; // pin of previous program change
unsigned char event_current_programchange[8]; // pin of current program change

unsigned char evnt0_MSB; // Status TYPE
unsigned char evnt0_LSB; // Status CHANNEL
unsigned char previous_evnt0; // previous status byte recieved
unsigned char previous_evnt1; // previous param1 byte recieved

unsigned char pin_mapped;  // used to hold the entry number in event map for the current pin.
unsigned char pin_mapto; // used in programing mode to assign an event_map entry to the current pin
unsigned char pin_select; // For programming mode. not always fixed buttons = pin, For banked buttons = (pin + (bank * DIN_BANKED_BUTTONS)) - DIN_BANKED_BUTTONS - DIN_FIXED_BUTTONS + 16.
unsigned char previous_pin_select; // as above but for the previous value

unsigned char bank; // current bank

unsigned char device_name[17]; // guess
unsigned short int count; // a counter used in various places
unsigned char countms;

unsigned short int bankstick_offset; // used sometimes when reading and writing banksrick
unsigned short int AIN_map_offset; // used sometimes when reading and writing banksrick

unsigned char current_programchange_evnt1[8]; // current program change event recieved or sent. for 8x banksticks

unsigned char AIN_fixed[8]; // gets values form bankstick. Tells whether or not AIN will change with program changes
unsigned char high_value; // the high value for the current pin_select
unsigned char low_value; // the low value for the current pin_select
unsigned char button_type; // momentary or switching for current pin_select
unsigned char bankstick_map[152]; // Holds channel # for each button/pedal., 1st 16 Fixed buttons, next 128 banked buttons, last 8 pedals
unsigned char bankstick_select;
unsigned char bankstick_channel[8];
unsigned int test; // I guess I was testing something
unsigned char channel;
unsigned char previous_entry;

///////////////////////////////////////////////////////////////////////////
// Definitions
//
///////////////////////////////////////////////////////////////////////////

// Program Mode
#define NONE          0
#define ENTER_SETUP   1
#define EVENT_SELECT  2
#define SET_MIN       3
#define SET_MAX       4
#define SET_AIN_FIXED 5
#define EXIT_SETUP    6
#define SET_BANKSTICK 7
#define SET_BANKSTICK1_CH 8
#define SET_BANKSTICK2_CH 9
#define SET_BANKSTICK3_CH 10
#define SET_BANKSTICK4_CH 11
#define SET_BANKSTICK5_CH 12
#define SET_BANKSTICK6_CH 13
#define SET_BANKSTICK7_CH 14
#define SET_BANKSTICK8_CH 15

// Event Types
#define PROGRAM_CHANGE   0
#define ON_OFF_ONLY      1
#define TAP_TEMPO        2
#define METER_ONLY       3
#define METER_OFF        4
#define METER_ON_OFF     5
#define METER_PAN        6
#define VALUE            7
#define USE_MAP_1        8
#define USE_MAP_2        9
#define USE_MAP_3        10
#define USE_MAP_4        11
#define USE_MAP_5        12
#define USE_MAP_6        13
#define USE_MAP_7        14
#define USE_MAP_8        15
#define USE_MAP_9        16
