/*
 * MIOS Pedal Box / Pedal Board - pbx.c
 * vbeta1.16 - September 2007
 * ==========================================================================
 *
 *  Copyright (C) 2007  Mick Crozier mcrozier@iinet.net.au
 *  Licensed for personal non-commercial use only.
 *  All other rights reserved.
 *
 * ==========================================================================
 */

#include "pbx.h"


////////////////////////////////////////////////////////////////////
// This function is called by MPROC_NotifyReceivedEvnt
// It scans the event_map It returns the match entry # if found otherwise 0xffff
//
// The event_map is uploaded to a bankstick (0x0000 to 0x12ed) by the pbx_device_upload application
//
// For the first entry
// name in 0x0000 to 0x000f (offest + 0 - 15)
// status byte in 0x0010 (offset + 16)
// param1 byte in 0x0011 (offset + 17)
// event_handler in 0x0012  (offset + 18)
// offset 19 is reserved for future use
//
// each loop adds 20 to get the start of the next entry (entry 2 name starts at 0x0014)
////////////////////////////////////////////////////////////////////
unsigned int scan_event_map(unsigned char evnt0, unsigned char evnt1)
{

  if (previous_evnt0 != evnt0 || previous_evnt1 != evnt1) {// find event only if it's differnt from previous, otherwise use existing found_event info. Save on read time
    evnt0_MSB = evnt0 >> 4;
    evnt0_MSB = evnt0_MSB << 4;
    evnt0_LSB = evnt0 - evnt0_MSB;

    for (t=0; t<9; t++) {  // set bankstick based on channel
      if (t == 8) {
        channel = 255;
        break;
      }
      if (bankstick_channel[t] == evnt0_LSB) {
        MIOS_BANKSTICK_CtrlSet(t);
        channel = bankstick_channel[t];
        break;
      }
    }
    if( channel < 16 ) {
      // reset variables
      bankstick_offset = 0x0000;

      //Scan event_map table for matching incoming events

      for (i=0; i<257; i++) {
        if (i == 256) return(0xffff); // used as 'not found' error code;
        if (evnt0_MSB == MIOS_BANKSTICK_Read(bankstick_offset + 16)) {
          if (evnt1 == MIOS_BANKSTICK_Read(bankstick_offset + 17)) { // these if statements are not linked to save read time
          //if found match copy to found_event_ variables
            previous_evnt0 = evnt0;
            previous_evnt1 = evnt1;
            previous_entry = i;
            for (t=0; t<16; t++) found_event.name[t] = MIOS_BANKSTICK_Read(bankstick_offset + t);
            found_event.status = (MIOS_BANKSTICK_Read(bankstick_offset + 16)) + evnt0_LSB;
            found_event.param1 = MIOS_BANKSTICK_Read(bankstick_offset + 17);
            found_event.event_handler = MIOS_BANKSTICK_Read(bankstick_offset + 18);
            break; //break for loop if found match
          } // evnt1
        }// evnt0
        bankstick_offset = bankstick_offset + 20;
      }// for loop
    } else {
      // error handling for no bankstick to go here.. will probably always be 'DO NOTHING'
      return(0xffff); // used as 'no bankstick' error code
    }

    return(i); //Returns the entry number

  } else {
    if (found_event.entry != previous_entry) {
      for (t=0; t<16; t++) found_event.name[t] = MIOS_BANKSTICK_Read(bankstick_offset + t);
      found_event.status = (MIOS_BANKSTICK_Read((found_event.entry * 20) + 16)) + evnt0_LSB;
      found_event.param1 = MIOS_BANKSTICK_Read((found_event.entry * 20) + 17);
      found_event.event_handler = MIOS_BANKSTICK_Read((found_event.entry * 20) + 18);
    }
    return(found_event.entry); // return previous used entry # if the event is the same as last recieved
  }
}

////////////////////////////////////////////////////////////////////
//These functions are called by notify midi and DIN in main.c
//They process the tap tempo feature
////////////////////////////////////////////////////////////////////
void tap_tempo_handler(void)
{
  if (app_flags.TAP_FLAG) { // tap has already been pressed
    tap_tempo_restart();
  } else {
    tap_tempo_start();
  }
}

void tap_tempo_start(void)
{
  MIOS_TIMER_Init(0x00, 10000); // first tap press
  app_flags.TAP_FLAG = 1;
}

void tap_tempo_restart(void)
{
  MIOS_TIMER_Stop();
  bpm = 60000 / bpmcountms; //  1 minute / (milliseconds between presses)
  bpmcountms = 0;
  MIOS_TIMER_Init(0x00, 10000);  // restart timer
}

void tap_tempo_stop(void) // times up for another press, stop the timer
{
  MIOS_TIMER_Stop();
  bpmcountms = 0;
  app_flags.TAP_FLAG = 0;
}

////////////////////////////////////////////////////////////////////
// This functions is called by in main.c
// It runs most of the program mode
//
// AIN_maps are located in the bankstick at 0x7000 - 0x7fff. Each map is 0x200 in size
// DIN_map is located in the bankstick at 0x6520 - 0x66cf
//
// Program mode 1 - Enter setup
// Program mode 2 - MIDI event select
// Program mode 3 - Low value select
// Program mode 4 - High value select
// Program mode 5 - AIN fixed edit
// Program mode 6 - Exit programing mode (display "saved")
////////////////////////////////////////////////////////////////////
void program_mode_handler(void)
{


  if (program_mode == SET_AIN_FIXED || program_mode >= SET_BANKSTICK1_CH) { // AIN_fixed programing mode
    count = 0;
    for (bankstick_offset=0x6900; bankstick_offset<0x6900 + 0x08; bankstick_offset++) {
      MIOS_BANKSTICK_Write(bankstick_offset, AIN_fixed[count]);
      count ++;
    }
    for (t=0; t<8; t++) {
      MIOS_BANKSTICK_CtrlSet(t);
      MIOS_BANKSTICK_Write(0x6ffe, bankstick_channel[t]);
    }
    program_mode = EXIT_SETUP;
    MIOS_LCD_MessageStop();
    app_flags.DISPLAY_UPDATE_REQ = 1;
  }

  else if (program_mode == EVENT_SELECT || program_mode == SET_MIN || program_mode == SET_MAX || program_mode == SET_BANKSTICK) { // 2 = saving pedal or button
    if (pin_select > 143) { // Programming a pedal
      if (MIOS_BANKSTICK_Read(0x0000 + (pin_mapto * 20) + 18) < METER_ONLY) { // can't map a pedal to program change, on/off, or tap
        MIOS_LCD_Clear();
        MIOS_LCD_CursorSet(0x00);  //First Line
        MIOS_LCD_PrintCString("Can not Map");
        program_mode = 2; // Stay in programing mode
      } else {
        MIOS_EEPROM_Write(pin_select, bankstick_select);
        bankstick_map[pin_select] = bankstick_select;
        pin_select = pin_select - 144;
        if (AIN_fixed[pin_select] == 0) {
          MIOS_BANKSTICK_CtrlSet(bankstick_select);
          MIOS_BANKSTICK_Write(0x7000 + (pin_select*0x200) + (current_programchange_evnt1[MIOS_BANKSTICK_CtrlGet()] * 4), pin_mapto); // save to eeprom
          MIOS_BANKSTICK_Write(0x7000 + (pin_select*0x200) + (current_programchange_evnt1[MIOS_BANKSTICK_CtrlGet()] * 4) + 1, low_value);
          MIOS_BANKSTICK_Write(0x7000 + (pin_select*0x200) + (current_programchange_evnt1[MIOS_BANKSTICK_CtrlGet()] * 4) + 2, high_value);
        } else {
          MIOS_BANKSTICK_CtrlSet(bankstick_select);
          MIOS_BANKSTICK_Write(0x7000 + (pin_select*0x200) + 0, pin_mapto); // save to eeprom
          MIOS_BANKSTICK_Write(0x7000 + (pin_select*0x200) + 1, low_value);
          MIOS_BANKSTICK_Write(0x7000 + (pin_select*0x200) + 2, high_value);
        }
        MIOS_AIN_DeadbandSet(AIN_DEADBAND);
        program_mode = EXIT_SETUP;  // Go to Exit program mode
        MIOS_LCD_MessageStop();
        app_flags.DISPLAY_UPDATE_REQ = 1;
      }
    }
    else {
      MIOS_BANKSTICK_CtrlSet(bankstick_select);
      MIOS_BANKSTICK_Write(0x6520 + (pin_select * 4) + 0, pin_mapto);
      MIOS_BANKSTICK_Write(0x6520 + (pin_select * 4) + 1, low_value);
      MIOS_BANKSTICK_Write(0x6520 + (pin_select * 4) + 2, high_value);
      MIOS_BANKSTICK_Write(0x6520 + (pin_select * 4) + 3, button_type);
      MIOS_EEPROM_Write(pin_select, bankstick_select);
      bankstick_map[pin_select] = bankstick_select;
      MIOS_AIN_DeadbandSet(AIN_DEADBAND);
      program_mode = EXIT_SETUP;  // Go to Exit program mode
      MIOS_LCD_MessageStop();
      app_flags.DISPLAY_UPDATE_REQ = 1;
    }
  }
  else if (program_mode == NONE) {  // Enter programing mode
    bankstick_select = bankstick_map[pin_select];
    MIOS_BANKSTICK_CtrlSet(bankstick_select);
    channel = MIOS_BANKSTICK_Read(0x6ffe);
    MIOS_AIN_DeadbandSet(5);
    program_mode = ENTER_SETUP;  // in the display handler, program mode is set to 2
    MIOS_LCD_MessageStop();
    app_flags.DISPLAY_UPDATE_REQ = 1;
  }
  else { // return to normal operation
    MIOS_AIN_DeadbandSet(7);  // Back to normal mode
    program_mode = NONE;
    MIOS_LCD_MessageStop();
    app_flags.DISPLAY_UPDATE_REQ = 1;
  }
}

////////////////////////////////////////////////////////////////////
// This function is called by notify AIN and DIN in main.c
// It transmits a midi message
////////////////////////////////////////////////////////////////////

void MIDI_message_TX(unsigned char evnt0, unsigned char evnt1, unsigned char evnt2, unsigned char channel)
{ // Sends a midi event based on the found_event variables

  evnt0_MSB = evnt0 >> 4; // These 2 lines force the channel to 0
  evnt0_MSB = evnt0_MSB << 4;

  evnt0_LSB = channel; // channel is controlled by bankstick

  MIOS_MIDI_TxBufferPut(evnt0_MSB + evnt0_LSB);
  MIOS_MIDI_TxBufferPut(evnt1);
  if (evnt0 != 0xc0) MIOS_MIDI_TxBufferPut(evnt2); // Don't send param2 if program change event
}


////////////////////////////////////////////////////////////////////
// These function is called in main.c
// They control the LED indicators
//
////////////////////////////////////////////////////////////////////
unsigned char current_value_hlp;

unsigned char dout_toggle(unsigned char evnt2, unsigned char low_value)
{
  if (evnt2 <= low_value) {
      dout_value = 0;
  } else {
      dout_value = 1;
  }
  return dout_value;
}
#if PEDALBOARD
char bank_pin_hlp; // holds the pin number of a banked entry (calculated with negative values)
unsigned char count_commands; // is the current Din_map entry number - 16
unsigned char which_bank; // bank number of the current DIN_map entry
unsigned char command_numbers_in_this_bank; // Flag holding the next Din_map entry that would be in the next bank
unsigned char current_bankstick;

void set_led_indicators(unsigned char bank)
{
  command_numbers_in_this_bank = DIN_BANKED_BUTTONS;
  which_bank = 1;
  count_commands = 0;
  current_bankstick = MIOS_BANKSTICK_CtrlGet(); // This will later be used to reset the bankstick to it's current #
  for (i=0; i<16 + 128; i++) { //is a fixed button
    MIOS_BANKSTICK_CtrlSet(MIOS_EEPROM_Read(i));
    t = MIOS_BANKSTICK_Read(0x6520 + (i * 4));
    current_value_hlp = event_current_value[t];
    if (i<DIN_FIXED_BUTTONS) {
      MIOS_DOUT_PinSet(i, current_value_hlp);
    } else if (i > 15) { // is a banked button
      count_commands++;
      if (count_commands == command_numbers_in_this_bank + 1) {
        command_numbers_in_this_bank = command_numbers_in_this_bank + DIN_BANKED_BUTTONS;
        which_bank++;
      }
      if (which_bank == bank) {
        bank_pin_hlp = ((bank * DIN_BANKED_BUTTONS) - DIN_BANKED_BUTTONS - (i - 16)); // could also use count_commands instead of i - 16
        bank_pin_hlp = 0 - bank_pin_hlp;
        MIOS_DOUT_PinSet(bank_pin_hlp + DIN_FIXED_BUTTONS, current_value_hlp);
      }
    }
  } // for loop
  MIOS_BANKSTICK_CtrlSet(current_bankstick); // Reset to current bankstick
}

#endif // PEDALBOARD

/////////////////////////////////////////////////////////////////////////////
// This is an assembly optimized function which scales a 7bit value between
// a minimum and maximum value... Thanks TK
/////////////////////////////////////////////////////////////////////////////
unsigned char Scale_7bit(unsigned char value, unsigned char min, unsigned char max)
{
  // scaled value is (<8-bit random> * ) >> 8
  PRODL = value << 1; // 8bit value

  if (min > max) {
    PRODH = min-max+1;  // range
  } else {
    PRODH = max-min+1;  // range
  }

__asm
    movf _PRODL, W
    mulwf _PRODH, 0
__endasm;


  if (min > max) {
    return max + PRODH;
  } else {
    return min + PRODH;
  }
}




