/*
 * MIOS Pedal Box / Pedal Board - pbx_lcd.c
 * vbeta1.16 - September 2007
 * ==========================================================================
 *
 *  Copyright (C) 2007  Mick Crozier mcrozier@iinet.net.au
 *  Licensed for personal non-commercial use only.
 *  All other rights reserved.
 *
 * ==========================================================================
 */
const unsigned char pan_map[16][17] = {
{"********        "},
{" *******        "},
{"  ******        "},
{"   *****        "},
{"    ****        "},
{"     ***        "},
{"      **        "},
{"       *        "},
{"        *       "},
{"        **      "},
{"        ***     "},
{"        ****    "},
{"        *****   "},
{"        ******  "},
{"        ******* "},
{"        ********"}
};

void visualise(void)   // Displays the gragh in the second line
{
  param2_value_reduced = (found_event.param2 >> 3);
  for (t=0; t<16; t++) {
    if (t<=param2_value_reduced)  {
      MIOS_LCD_PrintChar('*');
    } else {
      MIOS_LCD_PrintChar(' ');
    }
  }
}

 ///////////////////////////////////////////////////////////////////////////
 // Permanent display handler - called in main.c
 // Messages produced here stay on they screen untill overwritten
 ///////////////////////////////////////////////////////////////////////////

void display_handler(void) // for permanent display
{
// do nothing if no update has been requested
  if( !app_flags.DISPLAY_UPDATE_REQ )
    return;

    // clear request
    app_flags.DISPLAY_UPDATE_REQ = 0;

////////SETUP MODE

  if (program_mode == ENTER_SETUP) {

    MIOS_LCD_Clear();
    MIOS_LCD_CursorSet(0x00);  //First Line
    MIOS_LCD_PrintCString("Setup           ");
    program_mode = SET_BANKSTICK;
    MIOS_LCD_MessageStart(200);
  }

  else if (program_mode == EVENT_SELECT) {

    MIOS_LCD_CursorSet(0x00);  //First Line
  //Prints name
    if (pin_select > 143) {
      MIOS_LCD_PrintCString("Map Pedal ");
      MIOS_LCD_PrintBCD2(pin_select - 143);
      MIOS_LCD_PrintCString("    ");
    } else {
      MIOS_LCD_PrintCString("Map Button ");
      MIOS_LCD_PrintBCD3(pin_select + 1);
      MIOS_LCD_PrintCString("  ");
    }
    /*#if DIGITS_CONNECTED == 0
    MIOS_LCD_PrintCString(":B");
    MIOS_LCD_PrintBCD2(bank);
    #endif*/
    MIOS_LCD_CursorSet(0x40);  //Second Line
    for (t=0; t<16; t++) MIOS_LCD_PrintChar(MIOS_BANKSTICK_Read(0x0000 + (pin_mapto * 20) + t));

  }

  else if (program_mode == SET_MIN || program_mode == SET_MAX) {

    MIOS_LCD_CursorSet(0x00);  //First Line
    if (pin_select < 144) {
      MIOS_LCD_PrintCString("Low   B");
      MIOS_LCD_PrintBCD3(pin_select + 1);
    } else {
      MIOS_LCD_PrintCString("Low   P");
      MIOS_LCD_PrintBCD3(pin_select - 143);
    }
    MIOS_LCD_PrintCString("  High");
    MIOS_LCD_CursorSet(0x40);  //Second Line
    MIOS_LCD_PrintBCD3(low_value);
    //MIOS_LCD_PrintCString("     ");
    if (pin_select < 144) {
      if ( button_type == 0 ) {
        MIOS_LCD_PrintCString("  Switch  ");
      } else {
        MIOS_LCD_PrintCString("  Moment  ");
      }
    } else {
      MIOS_LCD_PrintCString("          ");
    }
    MIOS_LCD_PrintBCD3(high_value);
    if (program_mode == SET_MIN) {
      MIOS_LCD_CursorSet(0x43);
      MIOS_LCD_PrintChar('<');
    } else {
      MIOS_LCD_CursorSet(0x4c);
      MIOS_LCD_PrintChar('>');
    }
  }

  else if (program_mode == SET_AIN_FIXED) {

    MIOS_LCD_CursorSet(0x00);  //First Line
    MIOS_LCD_PrintCString("Fix Ped 12345678");
    MIOS_LCD_CursorSet(0x40);  //Second Line
    MIOS_LCD_PrintCString("Y=1 N=0 ");
    for (t=0; t<8; t++) MIOS_LCD_PrintBCD1(AIN_fixed[t]);
  }

  else if (program_mode == EXIT_SETUP) {

    MIOS_LCD_Clear();
    MIOS_LCD_CursorSet(0x00);  //First Line
    MIOS_LCD_PrintCString("Saved");
    MIOS_LCD_CursorSet(0x40);  //Second Line
    program_mode = 0;
    MIOS_LCD_MessageStart(200);
  }
  else if (program_mode == SET_BANKSTICK) {

    MIOS_LCD_CursorSet(0x00);  //First Line
    if (pin_select > 143) {
      MIOS_LCD_PrintCString("Pedal ");
      MIOS_LCD_PrintBCD2(pin_select - 143);
    } else {
      MIOS_LCD_PrintCString("Button ");
    }
    MIOS_LCD_PrintCString(" Ch:");
    MIOS_BANKSTICK_CtrlSet(bankstick_select);
    channel = MIOS_BANKSTICK_Read(0x6ffe);
    if( MIOS_BOX_STAT.BS_AVAILABLE ) {
      MIOS_LCD_PrintBCD2(channel + 1);
      MIOS_LCD_PrintCString("    ");
      MIOS_LCD_CursorSet(0x40);  //Second Line
      for (t=0; t<16; t++) MIOS_LCD_PrintChar(MIOS_BANKSTICK_Read(0x6800 + t));
    } else {
      MIOS_LCD_PrintCString("N/A  ");
      MIOS_LCD_CursorSet(0x40);
      MIOS_LCD_PrintCString("BS Not Available");
    }
  }
  else if (program_mode >= SET_BANKSTICK1_CH && program_mode <= SET_BANKSTICK4_CH) {

    MIOS_LCD_CursorSet(0x00);  //First Line
    MIOS_LCD_PrintCString("B.S   1  2  3  4");
    MIOS_LCD_CursorSet(0x40);  //Second Line
    MIOS_LCD_PrintCString("Ch. ");
    for (t=0; t<4; t++) {
      if (bankstick_channel[t] == 255) {
        MIOS_LCD_PrintCString(" NA");
      } else {
        MIOS_LCD_PrintBCD3(bankstick_channel[t] + 1);
      }
    }
    MIOS_LCD_CursorSet(0x44 + ((program_mode - SET_BANKSTICK1_CH) * 3));
    MIOS_LCD_PrintChar('>');
  }
  else if (program_mode >= SET_BANKSTICK5_CH && program_mode <= SET_BANKSTICK8_CH) {

    MIOS_LCD_CursorSet(0x00);  //First Line
    MIOS_LCD_PrintCString("B.S   5  6  7  8");
    MIOS_LCD_CursorSet(0x40);  //Second Line
    MIOS_LCD_PrintCString("Ch. ");
    for (t=4; t<8; t++) {
      if (bankstick_channel[t] == 255) {
        MIOS_LCD_PrintCString(" NA");
      } else {
        MIOS_LCD_PrintBCD3(bankstick_channel[t] + 1);
      }
    }
    MIOS_LCD_CursorSet(0x44 + ((program_mode - SET_BANKSTICK5_CH) * 3));
    MIOS_LCD_PrintChar('>');
  }

  else { //////////MIDI MODE (program changes only)

   MIOS_LCD_CursorSet(0x00);  //First Line
    if (event_previous_programchange != event_current_programchange) {
    // Print Program Change
      for (t=0; t<16; t++) MIOS_LCD_PrintChar(MIOS_BANKSTICK_Read(0x0000 + (event_current_programchange[MIOS_BANKSTICK_CtrlGet()] * 20) + t));
      MIOS_LCD_CursorSet(0x40);  //Second Line
      for (t=0; t<16; t++) MIOS_LCD_PrintChar(MIOS_BANKSTICK_Read(0x6800 + t));
    } else { // This is the main display screen
    // Show current program change status
      for (t=0; t<16; t++) MIOS_LCD_PrintChar(MIOS_BANKSTICK_Read(0x0000 + (event_previous_programchange[MIOS_BANKSTICK_CtrlGet()] * 20) + t));
      MIOS_LCD_CursorSet(0x40);  //Second Line
      for (t=0; t<16; t++) MIOS_LCD_PrintChar(MIOS_BANKSTICK_Read(0x6800 + t));
    }

    // Prints current bank if no LED digits are connected
    #if PEDALBOARD
    #if DIGITS_CONNECTED == 0
      MIOS_LCD_CursorSet(0x0b);  //First Line
      MIOS_LCD_PrintChar(':');
      MIOS_LCD_PrintChar('B');
      MIOS_LCD_PrintBCD2(bank);
    #endif
    #endif


/*
// for debug only
  MIOS_LCD_CursorSet(0x00);

  MIOS_LCD_PrintHex2(evnt0_LSB);
  MIOS_LCD_PrintHex2(evnt0_MSB);
*/
  } //End Not in Programing Mode
}

 ///////////////////////////////////////////////////////////////////////////
 // Temporary display handler - called in main.c
 // Messages produced here stay on they screen for 2 seconds or until overwritten
 ///////////////////////////////////////////////////////////////////////////

void display_temp(void)
{
  MIOS_LCD_CursorSet(0x00);  //First Line
  //Prints name
  for (t=0; t<16; t++) MIOS_LCD_PrintChar(found_event.name[t]);

  // Prints current bank if no LED digits are connected
  #if PEDALBOARD
  #if DIGITS_CONNECTED == 0
    MIOS_LCD_CursorSet(0x0b);  //First Line
    MIOS_LCD_PrintChar(':');
    MIOS_LCD_PrintChar('B');
    MIOS_LCD_PrintBCD2(bank);
  #endif
  #endif

  MIOS_LCD_CursorSet(0x40);  //Second Line

// On/Off Only
  if( found_event.event_handler == ON_OFF_ONLY) {

    if( found_event.param2 == high_value) {//cc value is 127
        MIOS_LCD_PrintCString("       On       ");
    }
    else if (found_event.param2 == low_value) {
        MIOS_LCD_PrintCString("       Off      ");
    }
  }

// Tap Tempo
  if( found_event.event_handler == TAP_TEMPO) {
    MIOS_LCD_PrintCString("  ");
    MIOS_LCD_PrintBCD5(bpm);
    MIOS_LCD_PrintCString(" BPM      ");
  }

// Just visualise
  if( found_event.event_handler == METER_ONLY) {
    visualise();
  }

// Visualise with 'off' when 0x00
  if( found_event.event_handler == METER_OFF) {
    if (found_event.param2 == low_value)  {
      MIOS_LCD_PrintCString("       Off      ");
    } else {
      visualise();
    }
  }

// visualise with 'on' at 0x7f and 'off' at 0x00
  if( found_event.event_handler == METER_ON_OFF ) {
    if( found_event.param2 == high_value) {//cc value is 127
      MIOS_LCD_PrintCString("       On       ");
    } else if (found_event.param2 == low_value) {
      MIOS_LCD_PrintCString("       Off      ");
    } else {
      visualise();
    }
  }

  if( found_event.event_handler == METER_PAN ) {
    param2_value_reduced = (found_event.param2 >> 3);
    if (found_event.param2 == 64) {
      MIOS_LCD_PrintCString("       ><       ");
    } else {
      for (t=0; t<16; t++) MIOS_LCD_PrintChar(pan_map[param2_value_reduced][t]);
    }
  }

  if( found_event.event_handler == VALUE ) {
    MIOS_LCD_PrintCString("      ");
    MIOS_LCD_PrintBCD3(found_event.param2);
    MIOS_LCD_PrintCString("       ");
  }

// Handler for value_maps
  if (found_event.event_handler >= USE_MAP_1) {
    switch (found_event.event_handler)
    {
      case USE_MAP_1: bankstick_offset = 0x1400;
            break;
      case USE_MAP_2: bankstick_offset = 0x1d02;
            break;
      case USE_MAP_3: bankstick_offset = 0x2604;
            break;
      case USE_MAP_4: bankstick_offset = 0x2f06;
            break;
      case USE_MAP_5: bankstick_offset = 0x3808;
            break;
      case USE_MAP_6: bankstick_offset = 0x410a;
            break;
      case USE_MAP_7: bankstick_offset = 0x4a0c;
            break;
      case USE_MAP_8: bankstick_offset = 0x530e;
            break;
      case USE_MAP_9: bankstick_offset = 0x5c10;
    }

    for (i=0; i<MIOS_BANKSTICK_Read(bankstick_offset); i++) {
      if (i == MIOS_BANKSTICK_Read(bankstick_offset) - 1) { // if it's the last entry!!
        if(found_event.param2 >= MIOS_BANKSTICK_Read(bankstick_offset + 2 + (i * 18) + 16)) {
      //if found match copy to found_event_ variables
          for (t=0; t<16; t++) MIOS_LCD_PrintChar(MIOS_BANKSTICK_Read(bankstick_offset + 2 + (i * 18) + t));
          break; //break for loop if found match
        }
      } else { // if not the last entry, get nearest lower value value if no exact match found
        if(found_event.param2 >= MIOS_BANKSTICK_Read(bankstick_offset + 2 + (i * 18) + 16) && found_event.param2 < MIOS_BANKSTICK_Read(bankstick_offset + 2 + ((i + 1) * 18) + 16)) {
          for (t=0; t<16; t++) MIOS_LCD_PrintChar(MIOS_BANKSTICK_Read(bankstick_offset + 2 + (i * 18) + t));
          break; //break for loop if found match
        }
      }
    }
  } // param_def > 5

#if DIGITS_CONNECTED
  digits_handler(bank);
#endif
#if PEDALBOARD
  set_led_indicators(bank);
#endif

  MIOS_LCD_MessageStart(255); // Display message for 2 seconds, then run LCD_init
}
