;setlevels.inc
;Simple.. Call Mid2Gain to read a variety of MIDI controls and create
;channel gain settings for the PGA volume array.
;
;Inputs: 	
;MIDI_VOL_0 A table of 16 MIDI volume settings, values from 0 to 127 
;MIDI_BAL_0	An optional table of MIDI Pan/Balance Settings, values from 1 to 127, with 64 as center
;master		An optional master volume register, values from 0 to 127
;
;outputs:
;CHANNEL_GAIN_0	A table of 32 volume settings, each valued from 0 to 255
;This tables values will be overwritten with each call to setlevels.
;Each level MAY then be decreased by calls to balance and mvol
;


#DEFINE center 64	;the center of volume/pan settings

Mid2Gain
;		BCF		OPTIONS,NewMidi
		CALL	setlevels
;		BTFSC	OPTIONS,Pan
		CALL	balance
;		BTFSC	OPTIONS,MVol
;		CALL	mvol
		RETURN


;simply double the MIDI volume (0-127) into both left and right channels
;_OR_ apply a log response curve, by calling LogByte
setlevels
		movlw	pairs				;how many pairs of audio?
		movwf	paircount			;our counter
		LFSR	FSR0,MIDI_VOL_0		;point to MIDI Volume tables
		LFSR	FSR1,CHANNEL_GAIN_0	;point to result tables
nxvolume
		MOVF	POSTINC0,W			;get MIDI volume (0-127)
;		RLNCF	WREG,F				;double it
		CALL	LogByte
		MOVWF	POSTINC1			;copy out to left channel
		MOVWF	POSTINC1			;and the right
		DECFSZ	paircount			;keep count
		BRA		nxvolume			;go back for more
		RETURN						;return when done

;Pan and Balance are handled the same.
;balance settings will never increase a channel volume,
;but only decrease the opposite side.
;if you pan Right, I will DECREASE the left channel gain
;If you pan Left, I will Decrease the right channel gain.
;Balance is centered at 64
;According to MIDI spec, a value of Zero is the same as value 1,
;both will be full left pan (right channel off)
balance
		movlw	pairs				;how many audio pairs to do
		MOVWF	paircount
		LFSR	FSR0,MIDI_BAL_0		;point to MIDI Balance Data 0-127, 64=Center
		LFSR	FSR1,CHANNEL_GAIN_0	; the gains we will affect
nxbal
		MOVLW	center
		CPFSLT	INDF0				;compare setting to center
		BRA		notleft				
		MOVF	INDF0,f				;zero setting = 1 according to MIDI standard
		BNZ		fixed
		INCF	INDF0				;adjust zeros to Ones
fixed
		MOVF	POSTINC1,F			;come here for pan left only, so we skip Left setting
		MOVF	POSTINC0,W			;value is 1 to 63 inclusive
		SUBLW	center				;value is now 63 to 1
		RLNCF	WREG,F				;value is now 126 to 2
		RLNCF	WREG,F				;value is now 252 to 4
		NEGF	WREG				;value is now 4 to 252
		MULWF	INDF1				;mult with current right chan gain setting
		MOVFF	PRODH,POSTINC1		;divide result by 256, put back in table
		DECFSZ	paircount			;done with this pair
		BRA		nxbal
		RETURN
notleft								;value is 64 to 127
		CPFSEQ	INDF0				;compare with center
		BNZ		panright
		MOVF	POSTINC0,F			;come here for centered pan
		MOVF	POSTINC1,F			;skip over without changes
		MOVF	POSTINC1,F
		DECFSZ	paircount			;done with this pair
		BRA		nxbal
		RETURN
panright							;value here is 65 to 127
		SUBWF	POSTINC0,W			;value is now 1 to 63
		RLNCF	WREG,F				;value is now 2 to 126
		RLNCF	WREG,F				;value is now 4 to 252
		NEGF	WREG				;value is now 252 to 4
		MULWF	INDF1				;mult with left gain setting
		MOVFF	PRODH,POSTINC1		;divide result by 256, put back in table
		MOVF	POSTINC1,F			;skip right channel
		DECFSZ	paircount			;done with this pair
		BRA		nxbal
		RETURN


;master volume may decrease levels by whatever the master setting is.
; valid range is 0 to 127. 0 will mute all channels, 127 will leave them almost untouched
; 64 will set everything at half it's current volume setting.
; Like balance, this can only Decrease settings, it will never increase them.
mvol
		MOVFF	pairs,paircount		;setup our counter
		LFSR	FSR1,CHANNEL_GAIN_0	;point to the levels table
		MOVF	master,W			;get our master setting (0-127)
		RLNCF	WREG				;double it (0-254)
nxmstr
		MULWF	INDF1				;multiply it by current left channel
		MOVFF	PRODH,POSTINC1		;divide result by 256 and put it back (Left)
		MULWF	INDF1				;multiply by current right channel
		MOVFF	PRODH,POSTINC1		;divide by 256 and put it back (Right)
		DECFSZ	paircount			;one less pair to do
		BRA		nxmstr				;are we done yet?
		RETURN
